==> hermes-5.6.0.0.rec2.tar.gz: hermes-5.6.0.0.rec2/spec/factories/applicants_factory.rb <==
# frozen_string_literal: true

require 'faker'
Faker::Config.locale = 'fr'

FactoryBot.define do
  factory:applicant, class: Hermes::Applicant do
    first_name { Faker::Name.first_name }
    last_name { Faker::Name.last_name }
    civility { ['M']}
    nationality { 'Française' }
    work_contract_type { 'CDI' }
    main_employer_siret { 'A0000000001234' }
    main_employer_name { 'Test Compagny' }
    workplace_insee_code { '34100' }
    actual_housing_details {{"is_house_owner": false}}
    postal_details {  { address_street: '10 Rue victor Hugo'}}
    birth_date { Faker::Date.between(70.years.ago, 31.years.ago) }
    year_minus_one { Date.today.year - 1}
    year_minus_two { Date.today.year - 2}

    transient do
      notifications_count { 0 }
      salary { nil }
      auto_set_user { true }
    end

    after :create do |applicant, evaluator|
      if evaluator.notifications_count.positive?
        create_list :notification, evaluator.notifications_count, targetable: applicant
      end

      #### Each 'unless' are used to keep original attributes if passed ###

      # Setup main_user by main_user_id or create it if not exist
      if evaluator.auto_set_user
        applicant.user = User.find(applicant.main_user_id) if applicant.user.nil? && applicant.main_user_id.present?
        applicant.user = FactoryBot.create(:user, auth_roles: ['demandeur']) unless applicant.user
        applicant.main_user_id = applicant.user.id
        applicant.email =  applicant.user.email if applicant.email.blank?
      end

      applicant.birth_plus_30 = applicant.birth_date + 30.years unless applicant.birth_plus_30 || applicant.birth_date.nil?

      # Auto compute last years salaries if transiant salary is passed (applicant earns 5% more each years)
      if evaluator.salary
        # We deduct 5% less from last year's annual salary compared to the current salary.
        applicant.amount_year_minus_one = ((evaluator.salary * 12) / 1.05) unless applicant.amount_year_minus_one
        # We deduct 10% less from last year's annual salary compared to the current salary.
        applicant.amount_year_minus_two = ((evaluator.salary * 12) / 1.1)  unless applicant.amount_year_minus_two
        applicant.monthly_incomes = { 
          SAL: evaluator.salary,
          RET: 0,
          CHO: 0,
          PAR: 0,
          PINV: 0,
          AF: 0,
          AAH: 0,
          AEEH: 0,
          AJPP: 0,
          RSA: 0,
          AMV: 0,
          PAJE: 0,
          BE: 0,
          AUT: 0,
          PDA: 0 
        } unless applicant.monthly_incomes
      end
      applicant.save!
    end

    trait :with_eligible_offers do
      # This trait is used to auto generale eligible offers for applicant informations
      nationality { (1..10).to_a.sample.to_s }
      email { Faker::Internet.email }
      homephone { Faker::PhoneNumber.phone_number }
      civility { (1..3).to_a.sample.to_s }
      birth_date { Faker::Date.between(70.years.ago, 19.years.ago) }
      main_employer_name { Faker::Company.name }
      work_contract_type { 'CDI' }
      main_employer_siret { Faker::Company.french_siret_number }
      workplace_postal_code { Faker::Address.zip_code }
      amount_year_minus_one { 31200.0 }
      amount_year_minus_two { 31200.0 }
      monthly_charges do
        {
          PAV: 0
        }
      end
      name_will_change { false }
      email_will_change { false }

      transient do
        offers_count { 1 }
        maried { true }
        salary { 1300 }
        spouse_salary { 1300 }
        pacted { false }
        children_count { 0 }
        live_with_childs { true }
      end
  
      after(:create) do |applicant, evaluator|
        rent_amount = (evaluator.salary / 4)
        rent_with_charges = (rent_amount * 1.2)
        rental_charges = (rent_with_charges - rent_amount)
        total_charges = rental_charges
        family_size = 0
        dependents = []

        if evaluator.maried || evaluator.pacted
          family_size += 1
          co_applicant = create(:applicant)
          rel_type = evaluator.pacted ? 'Pacsé (e)' : 'Marié (e)'
          dependents.push({
            id: co_applicant.id,
            last_name: co_applicant.last_name, 
            first_name: co_applicant.first_name, 
            birth_date: co_applicant.birth_date,
            rel_type: rel_type,
            sex: %w[M F].sample
          })
        end

        evaluator.children_count.times do
          family_size += 1
          child = create(:applicant, birth_date: Faker::Date.between(17.years.ago, 1.years.ago))
          rel_type = evaluator.live_with_childs ? 'enfant à charge' : 'droit de visite'
          dependents.push({
            id: child.id,
            last_name: child.last_name, 
            first_name: child.first_name, 
            birth_date: child.birth_date,
            rel_type: rel_type,
            sex: %w[M F].sample
          })
        end

        # Create eligibles offers
        typology = family_size + 1
        typology = 6 if typology >= 6
        typology = "T#{typology}"

        ceil_category_1 = (applicant.amount_year_minus_two * 1.1)

        housing_offers = create_list(:housing_offer, evaluator.offers_count, {
          typology: typology,
          insee_code: "34000",
          postal_code: "34000",
          rent_amount: rent_amount,
          rent_with_charges: rent_with_charges,
          surface: (80..200).to_a.sample,
          rooms: [2..8].sample,
          availability_date: Faker::Date.backward,
          description: Faker::Lorem.sentence,
          reserved: false,
          rental_charges: rental_charges,
          lessor_status: "PUBLIE",
          offer_status: "LOGEMENT PUBLIE",
          ceil_category_1: ceil_category_1,
          ceil_category_2: (ceil_category_1 * 2),
          ceil_category_3: (ceil_category_1 * 3),
          ceil_category_4: (ceil_category_1 * 4),
          ceil_category_5: (ceil_category_1 * 5),
          ceil_category_6: (ceil_category_1 * 6),
          ceil_category_7: (ceil_category_1 * 7),
          ceil_category_8: (ceil_category_1 * 8),
          ceil_category_9: (ceil_category_1 * 9),
          ceil_category_10: (ceil_category_1 * 10),
          ceil_category_11: (ceil_category_1 * 11),
          ceil_category_12: (ceil_category_1 * 12),
          floor_category_1: 0.0,
          floor_category_2: 0.0,
          floor_category_3: 0.0,
          floor_category_4: 0.0,
          floor_category_5: 0.0,
          floor_category_6: 0.0,
          floor_category_7: 0.0,
          floor_category_8: 0.0,
          floor_category_9: 0.0,
          floor_category_10: 0.0,
          floor_category_11: 0.0,
          floor_category_12: 0.0,
          min_revenues: (evaluator.salary * 0.8),
          total_charges: total_charges,
          min_effort_rate: 0.333,
          administration_area: "Grand Centre",
          lessor_name: Faker::Company.name,
          lessor_siren: Faker::Company.french_siren_number,
          entities_siret: [applicant.main_employer_siret],
          designation_date_end: Faker::Date.forward,
          commercialized: false,
          is_mad: true,
          is_mad_updated_at: (DateTime.current - 1.day),
        })

        # Create HosuingRequest to make eligibility
        create(
          :housing_request,
          user_id: applicant.user.id,
          is_active: true, 
          priority_points_indexes: [], 
          has_priority: false,
          reference_tax_revenues: applicant.amount_year_minus_two,
          dependents: dependents, 
          category_type: 3,
          single_parent: false,
          widen_request: true,
          housing_type_list: ["RE"],
          seeked_district_postal_codes: housing_offers.map(&:postal_code).uniq, 
          seeked_departments: housing_offers.map(&:postal_code).uniq.map { |zip_code| zip_code[0..1] }, 
          seeked_localisation_list: housing_offers.map(&:postal_code).map do |zip_code|
            {
              nom: Faker::Address.city,
              code: zip_code,
              codeDepartement: zip_code[0..1],
              codesPostaux: [zip_code],
              population: rand(10000..1000000),
              _score: [1, rand].sample
            }
          end,
          requester_firstname: applicant.first_name, 
          requester_lastname: applicant.last_name, 
          requester_fullname: applicant.full_name, 
          requester_email: applicant.email,
          requester_birth_date: applicant.birth_date,
          monthly_revenues: evaluator.salary,
          unborn_children: [{}], 
          actual_housing_details: {}, 
          postal_details: {}, 
          owned_housing_details: {
            type: nil,
            postal_code: nil,
            district: nil
          }, 
          employee_id: "MLLS", 
          employee_start_date: DateTime.current - 3.years,
          entity_access_level: "total",
          tab_state: "0", 
          motivations: ["02"],
          seeked_housing_categories: [""], 
          min_surface: 25, 
          dependents_monthly_incomes: {}, 
          dependents_monthly_charges: {}, 
          applications_nb: 0,
          name_has_changed: false,
          birth_date_has_changed: false,
          monthly_revenues_will_change: false,
          category_may_change: false,
          should_update_eligibility: false,
          tax_revenues_will_change: false,
          housing_offer_ids: [],
          entity_status: "A VALIDER",
          request_status: "ELIGIBLE", 
          nud_creations: {},
          main_applicant_id: applicant.id, 
          applicant_ids: dependents.map { |dep| dep['id'] },
        ) if evaluator.offers_count >= 1
      end
    end
  end
end

==> hermes-6.0.0.6.rec2.tar.gz: hermes-6.0.0.6.rec2/spec/factories/applicants_factory.rb <==
# frozen_string_literal: true

require 'faker'
Faker::Config.locale = 'fr'

FactoryBot.define do
  factory:applicant, class: Hermes::Applicant do
    first_name { Faker::Name.first_name }
    last_name { Faker::Name.last_name }
    civility { ['M']}
    nationality { 'Française' }
    work_contract_type { 'CDI' }
    main_employer_siret { 'A0000000001234' }
    workplace_insee_code { '34100' }
    actual_housing_details {{"is_house_owner": false}}
    birth_date { Faker::Date.between(70.years.ago, 31.years.ago) }
    year_minus_one { Date.today.year - 1}
    year_minus_two { Date.today.year - 2}
    postal_details {  { address_street: '10 Rue victor Hugo'}}
    main_employer_name { 'COMPAGNY' }

    transient do
      notifications_count { 0 }
      salary { nil }
      auto_set_user { true }
    end

    after :create do |applicant, evaluator|
      if evaluator.notifications_count.positive?
        create_list :notification, evaluator.notifications_count, targetable: applicant
      end

      #### Each 'unless' are used to keep original attributes if passed ###

      # Setup main_user by main_user_id or create it if not exist
      if evaluator.auto_set_user
        applicant.user = User.find(applicant.main_user_id) if applicant.user.nil? && applicant.main_user_id.present?
        applicant.user = FactoryBot.create(:user, auth_roles: ['demandeur']) unless applicant.user
        applicant.main_user_id = applicant.user.id
        applicant.email =  applicant.user.email if applicant.email.blank?
      end

      applicant.birth_plus_30 = applicant.birth_date + 30.years unless applicant.birth_plus_30 || applicant.birth_date.nil?

      # Auto compute last years salaries if transiant salary is passed (applicant earns 5% more each years)
      if evaluator.salary
        # We deduct 5% less from last year's annual salary compared to the current salary.
        applicant.amount_year_minus_one = ((evaluator.salary * 12) / 1.05) unless applicant.amount_year_minus_one
        # We deduct 10% less from last year's annual salary compared to the current salary.
        applicant.amount_year_minus_two = ((evaluator.salary * 12) / 1.1)  unless applicant.amount_year_minus_two
        applicant.monthly_incomes = { 
          SAL: evaluator.salary,
          RET: 0,
          CHO: 0,
          PAR: 0,
          PINV: 0,
          AF: 0,
          AAH: 0,
          AEEH: 0,
          AJPP: 0,
          RSA: 0,
          AMV: 0,
          PAJE: 0,
          BE: 0,
          AUT: 0,
          PDA: 0 
        } unless applicant.monthly_incomes
      end
      applicant.save!
    end

    trait :with_eligible_offers do
      # This trait is used to auto generale eligible offers for applicant informations
      nationality { (1..10).to_a.sample.to_s }
      email { Faker::Internet.email }
      homephone { Faker::PhoneNumber.phone_number }
      civility { (1..3).to_a.sample.to_s }
      birth_date { Faker::Date.between(70.years.ago, 19.years.ago) }
      main_employer_name { Faker::Company.name }
      work_contract_type { 'CDI' }
      main_employer_siret { Faker::Company.french_siret_number }
      workplace_postal_code { Faker::Address.zip_code }
      amount_year_minus_one { 31200.0 }
      amount_year_minus_two { 31200.0 }
      monthly_charges do
        {
          PAV: 0
        }
      end
      name_will_change { false }
      email_will_change { false }

      transient do
        offers_count { 1 }
        maried { true }
        salary { 1300 }
        spouse_salary { 1300 }
        pacted { false }
        children_count { 0 }
        live_with_childs { true }
      end
  
      after(:create) do |applicant, evaluator|
        rent_amount = (evaluator.salary / 4)
        rent_with_charges = (rent_amount * 1.2)
        rental_charges = (rent_with_charges - rent_amount)
        total_charges = rental_charges
        family_size = 0
        dependents = []

        if evaluator.maried || evaluator.pacted
          family_size += 1
          co_applicant = create(:applicant)
          rel_type = evaluator.pacted ? 'Pacsé (e)' : 'Marié (e)'
          dependents.push({
            id: co_applicant.id,
            last_name: co_applicant.last_name, 
            first_name: co_applicant.first_name, 
            birth_date: co_applicant.birth_date,
            rel_type: rel_type,
            sex: %w[M F].sample
          })
        end

        evaluator.children_count.times do
          family_size += 1
          child = create(:applicant, birth_date: Faker::Date.between(17.years.ago, 1.years.ago))
          rel_type = evaluator.live_with_childs ? 'enfant à charge' : 'droit de visite'
          dependents.push({
            id: child.id,
            last_name: child.last_name, 
            first_name: child.first_name, 
            birth_date: child.birth_date,
            rel_type: rel_type,
            sex: %w[M F].sample
          })
        end

        # Create eligibles offers
        typology = family_size + 1
        typology = 6 if typology >= 6
        typology = "T#{typology}"

        ceil_category_1 = (applicant.amount_year_minus_two * 1.1)

        housing_offers = create_list(:housing_offer, evaluator.offers_count, {
          typology: typology,
          insee_code: "34000",
          postal_code: "34000",
          rent_amount: rent_amount,
          rent_with_charges: rent_with_charges,
          surface: (80..200).to_a.sample,
          rooms: [2..8].sample,
          availability_date: Faker::Date.backward,
          description: Faker::Lorem.sentence,
          reserved: false,
          rental_charges: rental_charges,
          lessor_status: "PUBLIE",
          offer_status: "LOGEMENT PUBLIE",
          ceil_category_1: ceil_category_1,
          ceil_category_2: (ceil_category_1 * 2),
          ceil_category_3: (ceil_category_1 * 3),
          ceil_category_4: (ceil_category_1 * 4),
          ceil_category_5: (ceil_category_1 * 5),
          ceil_category_6: (ceil_category_1 * 6),
          ceil_category_7: (ceil_category_1 * 7),
          ceil_category_8: (ceil_category_1 * 8),
          ceil_category_9: (ceil_category_1 * 9),
          ceil_category_10: (ceil_category_1 * 10),
          ceil_category_11: (ceil_category_1 * 11),
          ceil_category_12: (ceil_category_1 * 12),
          floor_category_1: 0.0,
          floor_category_2: 0.0,
          floor_category_3: 0.0,
          floor_category_4: 0.0,
          floor_category_5: 0.0,
          floor_category_6: 0.0,
          floor_category_7: 0.0,
          floor_category_8: 0.0,
          floor_category_9: 0.0,
          floor_category_10: 0.0,
          floor_category_11: 0.0,
          floor_category_12: 0.0,
          min_revenues: (evaluator.salary * 0.8),
          total_charges: total_charges,
          min_effort_rate: 0.333,
          administration_area: "Grand Centre",
          lessor_name: Faker::Company.name,
          lessor_siren: Faker::Company.french_siren_number,
          entities_siret: [applicant.main_employer_siret],
          designation_date_end: Faker::Date.forward,
          commercialized: false,
          is_mad: true,
          is_mad_updated_at: (DateTime.current - 1.day),
        })

        # Create HosuingRequest to make eligibility
        create(
          :housing_request,
          user_id: applicant.user.id,
          main_user_id: applicant.user.id,
          is_active: true, 
          priority_points_indexes: [], 
          has_priority: false,
          reference_tax_revenues: applicant.amount_year_minus_two,
          dependents: dependents, 
          category_type: 3,
          single_parent: false,
          widen_request: true,
          housing_type_list: ["RE"],
          seeked_district_postal_codes: housing_offers.map(&:postal_code).uniq, 
          seeked_departments: housing_offers.map(&:postal_code).uniq.map { |zip_code| zip_code[0..1] }, 
          seeked_localisation_list: housing_offers.map(&:postal_code).map do |zip_code|
            {
              nom: Faker::Address.city,
              code: zip_code,
              codeDepartement: zip_code[0..1],
              codesPostaux: [zip_code],
              population: rand(10000..1000000),
              _score: [1, rand].sample
            }
          end,
          requester_firstname: applicant.first_name, 
          requester_lastname: applicant.last_name, 
          requester_fullname: applicant.full_name, 
          requester_email: applicant.email,
          requester_birth_date: applicant.birth_date,
          monthly_revenues: evaluator.salary,
          unborn_children: [{}], 
          actual_housing_details: {}, 
          postal_details: {}, 
          owned_housing_details: {
            type: nil,
            postal_code: nil,
            district: nil
          }, 
          employee_id: "MLLS", 
          employee_start_date: DateTime.current - 3.years,
          entity_access_level: "total",
          tab_state: "0", 
          motivations: ["02"],
          seeked_housing_categories: [""], 
          min_surface: 25, 
          dependents_monthly_incomes: {}, 
          dependents_monthly_charges: {}, 
          applications_nb: 0,
          name_has_changed: false,
          birth_date_has_changed: false,
          monthly_revenues_will_change: false,
          category_may_change: false,
          should_update_eligibility: false,
          tax_revenues_will_change: false,
          housing_offer_ids: [],
          entity_status: "A VALIDER",
          request_status: "ELIGIBLE", 
          nud_creations: {},
          main_applicant_id: applicant.id, 
          applicant_ids: dependents.map { |dep| dep['id'] },
        ) if evaluator.offers_count >= 1
      end
    end
  end
end
